#!/usr/local/bin/perl 

require 'heald_lib.pl';
use Getopt::Std;

getopts(':HWA12345');

if ($opt_H) {$act='H'};
if ($opt_W) {$act='W'};
if ($opt_A) {$act='A'};
die "action not recognised\n" unless ($act);

if ($opt_1) {$list=1};
if ($opt_2) {$list=2};
if ($opt_3) {$list=3};
if ($opt_4) {$list=4};
if ($opt_5) {$list=5};

die "list type not recognised\n" unless ($list);

@hashlist = \(%child,%day,%month,%year,%note,
		%father,%mother,%place,%parish,
		%county,%chapman,%batch,%lds);

###############  BUILD DATABASE ################################

$| = 1;
my $ct = 0;
my $latest=0;
$mo_fname{$latest} = '__NULL__';
while (<>) {
	next if (/^#/);

	$ct++;
	($child,$day,$month,$year,$note,
		$father,$mother,$place,$parish,
		$county,$chapman,$batch,$lds) = @line = split(/:/);

	# next if ($year < 1801);
	next if (($child=~ /^d\./) || ($child=~ /^EMIGRATED/)); 
	# next if ($lds =~ #//\s*m\.#);
	$line = $_;
	
	#next unless ($mother && $father);

	($ch_sname{$ct},$ch_fname{$ct}) = names(FULLNAME => $child,SEX => MF);
	($mo_sname{$ct},$mo_fname{$ct}) = names(FULLNAME => $mother,SEX => F);
	($fa_sname{$ct},$fa_fname{$ct}) = names(FULLNAME => $father,SEX => M);
	
	#print "$fa_sname{$ct}:$fa_fname{$ct} = $mo_sname{$ct}:$mo_fname{$ct}\n";
	
	# remove 'Mrs John Heald' surnames/forenames
	if (($fa_sname{$ct} eq 'HEALD') && ($mo_sname{$ct} eq 'HEALD'))  
	{
		unless(   ($batch =~ /^M/)    # marriage register
		        &&(!(do{
					my ($m1,$m1s,$m1f);
					($m1 = $mother) =~ s/\[[\w\s\/]*\]//;
					($m1s,$msf) = names(FULLNAME => $m1,SEX => F);
					$mls;
				  }))
		){
			$mo_sname{$ct} = '';
			if ($fa_fname{$ct} eq ($std_boys{$mo_fname{$ct}} || $mo_fname{$ct})) {
				$mo_fname{$ct} = '';
				next;
			}
		}
	}

	if ($act eq 'W') {next unless (($fa_sname{$ct} eq "HEALD") || ($ch_sname{$ct} eq "HEALD")) };
	if ($act eq 'H') {next unless ($mo_sname{$ct} eq "HEALD")};
	if ($act eq 'A') {next unless (($fa_sname{$ct} eq "HEALD") || ($mo_sname{$ct} eq "HEALD"))};

	# next unless ($fa_fname{$ct} && $mo_fname{$ct});

	if (no_conflict($ct,$latest)) {
		next unless 
			(		($mo_fname{$ct}) && (!$mo_fname{$latest})
			||  # can fill in mother's surname
				(	($mo_sname{$ct} && (!$mo_sname{$latest}))
				&& 	(($mo_fname{$ct}) || (!$mo_fname{$latest}))
				)
			||  # can fill in more accurate date, without losing surname
				(	(   (   ($month{$latest} =~ /^(Bef).?$/)
				        && 	($month !~ /^(Bef).?$/) )
					||  (   ($month{$latest} =~ /^(Abt).?$/)
				        && 	($month !~ /^(Abt|Bef).?$/) )
					)
				&& 	(($mo_sname{$ct}) || (!$mo_sname{$latest}))
				&& 	(($mo_fname{$ct}) || (!$mo_fname{$latest}))
				)	
			||  # marriage register, without losing surname
				(	($batch =~ /^M/)
				&& 	(($mo_sname{$ct}) || (!$mo_sname{$latest}))
				&& 	(($mo_fname{$ct}) || (!$mo_fname{$latest}))
				)
			);
		#remove previous entry
		delete $mark{$latest};
	}
	
	
	# add to marked records
	$latest = $ct;
	$mark{$latest}=1;
	$line{$latest} = $line;
	$date{$latest} = $year + $mindex{$month}/13.0 + $day/404.0;
	if ($fa_sname{$latest} eq "HEALD") {
		$spouse{$latest} = $mo_sname{$latest};
	} else {
		$spouse{$latest} = $fa_sname{$latest};
	}
	foreach my $rhash (@hashlist) {
		$rhash->{$latest} = shift(@line);
	}
	# NOTE 
	# sadly, no whole array dereferencing @data-> or @data 
	# bizarrely, $rhash doesn't work either
}

###############  SORT DATABASE ################################

&read_counties;

sub by_ordering_local {
	 by_county() ||
	 by_fa_sname() || by_fa_fname() || by_mo_fname() || by_date();
}

sub by_his_firstname {
	 by_fa_fname2() || by_date();
}

sub by_his_firstname_then_hers {
	 by_fa_fname2() || by_mo_fname2() || by_date();
}

sub by_his_surname {
	 by_fa_sname2() || by_date();
}

sub by_his_s1name {
	 by_fa_s1name2() || by_date();
}

#sub by_ordering {
#	 by_fa_sname() || by_fa_fname() || by_mo_fname() || by_date();
#}

sub by_her_firstname {
	 by_mo_fname2() || by_date();
}

sub by_her_firstname_then_his {
	 by_mo_fname2() || by_fa_fname2() || by_date();
}

sub by_her_surname {
	 by_mo_sname2() || by_date();
}

sub by_her_s1name {
	 by_mo_s1name2() || by_date();
}

sub by_spouse_name {
	(unblank($spouse{$a}) cmp unblank($spouse{$b})) || by_date();
}
	
sub list_by_his_firstname {
	foreach $ct (sort by_his_firstname (keys(%mark))) {
		$ini = initial($fa_fname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n"; 		
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

sub list_by_his_firstname_then_hers {
	foreach $ct (sort by_his_firstname_then_hers (keys(%mark))) {
		$ini = initial($fa_fname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}
	
sub list_by_his_surname {
	foreach $ct (sort by_his_surname (keys(%mark))) {
		$ini = initial($fa_sname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n"; 		
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

sub list_by_his_s1name {
	foreach $ct (sort by_his_s1name (keys(%mark))) {
		$ini = initial($fa_sname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

sub list_by_her_firstname {
	foreach $ct (sort by_her_firstname (keys(%mark))) {
		$ini = initial($mo_fname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 
		print "$line{$ct}";
		$prev=$ct;
	}
}

sub list_by_her_firstname_then_his {
	foreach $ct (sort by_her_firstname_then_his (keys(%mark))) {
		$ini = initial($mo_fname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 
		print "$line{$ct}";
		$prev=$ct;
	}
}

sub list_by_her_surname {
	foreach $ct (sort by_her_surname (keys(%mark))) {
		$ini = initial($mo_sname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

sub list_by_her_s1name {
	foreach $ct (sort by_her_s1name (keys(%mark))) {
		$ini = initial($mo_sname{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

sub list_by_spouse_surname {
	foreach $ct (sort by_spouse_name (keys(%mark))) {
		$ini = initial($spouse{$ct});
		if (initial($ini) ne $last_ini) {
			print "#\n# -- $ini -- \n#\n";
			$last_ini = $ini;
		} 		
		print $line{$ct};
		$prev=$ct;
	}
}

if ($act eq 'W') {
	if ($list==1) {&list_by_his_firstname};
	if ($list==2) {&list_by_his_firstname_then_hers};
	if ($list==3) {&list_by_her_firstname};
	if ($list==4) {&list_by_her_surname};
	if ($list==5) {&list_by_her_s1name};
}
if ($act eq 'H') {
	if ($list==1) {&list_by_her_firstname};
	if ($list==2) {&list_by_her_firstname_then_his};
	if ($list==3) {&list_by_his_firstname};
	if ($list==4) {&list_by_his_surname};
	if ($list==5) {&list_by_his_s1name};
}

if ($act eq 'A') {
	&list_by_spouse_surname;
}

__END__



