#!/usr/local/bin/perl 

my $DATA_DIR	=  '../code/dat';
my $BOYS_NAMES 	=  "$DATA_DIR/boys.dat";
my $GIRLS_NAMES =  "$DATA_DIR/girls.dat";
my $SURNAMES	=  "$DATA_DIR/surname_vs.dat";

my $COUNTIES	=  "$DATA_DIR/counties.dat";

my %AREAS= (LAN => "$DATA_DIR/areas_lan.dat",
			YKS => "$DATA_DIR/areas_yks.dat");
			
sub std{
	my ($word,$case) = @_;
	for ($word) {
		chomp;
		s/^\s*//;
		s/_*\.*\s*$//;
	}
	for ($case) {
		/^[L|l]/	&& return(lc($word));	# lower case
		/^[M|m]/	&& return($word);		# mixed case
		/^[U|u]/	&& return(uc($word));	# upper case
	}
	return ($word);				# default: mixed case
}
	
# Read in a data-hash from a colon-separated file

sub read_hash {
	my %args = (
		FILENAME 	=> '',
		HASH		=> '',
		CASE	 	=> 'M',
		FIRSTKEY	=> 0,
		NKEYS_MAX		=> 0,
		VALFIELD 	=> 0,
		VAL_CASE	=> '',
		DEBUG		=> 0,
		@_,
	);
	open(FILE,$args{FILENAME})  
			|| die "couldn't open file $args{FILENAME}\n";
	while (<FILE>) {
		my @items = split(/:/);
		my $val = $items[$args{VALFIELD}];
		my $std_val = std($val,$args{VAL_CASE} || $args{CASE});
		my @keys;
		if (!$args{NKEYS_MAX}) {
		 	@keys = splice(@items,$args{FIRSTKEY});
		} else {
			@keys = splice(@items,$args{FIRSTKEY},$args{NKEYS_MAX});
		}
		foreach my $key (@keys) {
			my $std_key = std($key,$args{CASE});
			$args{HASH}->{$std_key} = $std_val;
			if ($args{DEBUG}) {
				print "\t$std_key -> $args{HASH}->{$std_key}\n";
			}
		}
	}
	close(FILE);
}

sub read_counties {
	read_hash(FILENAME => $COUNTIES, HASH => \%cindex,
				FIRSTKEY => 0, NKEYS_MAX => 1, VALFIELD => 1);
	read_hash(FILENAME => $COUNTIES, HASH => \%county_name, 
				FIRSTKEY => 0, NKEYS_MAX => 1, VALFIELD => 2);
}


sub read_name_variants {
	read_hash(FILENAME => $BOYS_NAMES, HASH => \%std_boys,
						CASE => 'L'); 
	read_hash(FILENAME => $GIRLS_NAMES, HASH => \%std_girls,
						CASE => 'L'); 
	read_hash(FILENAME => $SURNAMES, HASH => \%std_surnames, 
						CASE => 'U'); 
}

#  return surname,forenames
#  standardised forms, in lower case
{
my $init = 0;
sub names {
	my %args = (
		FULLNAME	=> '',
		SEX			=> 'MF',
		@_,
	);
	if (!$init) {&read_name_variants;$init=1};
	my ($title,$fnames,undef,$sname) = 
		($args{FULLNAME} =~ m#(Mrs|Mr|Miss)?\.?\s*([\w\s\[\]\.]*)(/([\w\s\.]+)/)?#);
	#print "$fname\n";
	$fnames =~ s/[_\]\[\.]/ /g;
	$fnames =~ tr/[A-Z]/[a-z]/;
	my @fnames = split(' ',$fnames);
	if ($args{SEX} =~ /M/ ) {
		$fnames[0] = $std_boys{$fnames[0]} || $fnames[0];
	}
	if ($args{SEX} =~ /F/ ) {
		$fnames[0] = $std_girls{$fnames[0]} || $fnames[0];
	}
	#print "\t$sname -> $std_surnames{$sname}\n";
	$sname = $std_surnames{$sname} || $sname;
	return($sname,@fnames);
}
}

sub std_parish{
 	my ($parish,undef) = @_;
	my ($par,@par);
	for ($parish) {
		tr/A-Z/a-z/;
		s/<?(of )?>?//g;
		s/\[?\]?//g;
		s/^\s*//;
	}
	return ($parish);
}

{
my $debug =0;
my $have_areas =0;
sub read_areas {
	foreach my $county (keys %AREAS) {
		open(AREAS,$AREAS{$county})
			|| die "couldn't open $county areas file, $AREAS{$county}\n";
		while (<AREAS>) {
		    next if /^#/;
		    chomp;
		    my ($index,$name) = split(/:/);
			my $par = std_parish($name);
			$area_index{$county}{$par} = $index;
			if ($debug) {
				print "\t$name -> $par -> $area_index{$county}{$par}\n";
			}
		}
	}
	$have_areas = 1;
}

sub area{
	if (!$have_areas) {&read_areas};
	my ($ct,undef) = @_;
	my $chap = $chapman{$ct};
	return(0) unless ($area_index{$chap});
	my $par = std_parish($parish{$ct});
	#print "\t$parish{$ct} -> $par -> $area_index{$chap}{$par}\n";
	return ($area_index{$chap}{$par} || 999);
}
}

%mindex = (
	'Abt.'	=> 0,
	Jan		=> 1,
	Feb		=> 2,
	Mar		=> 3,
	Apr		=> 4,
	May		=> 5,
	Jun		=> 6,
	Jul		=> 7,
	Aug		=> 8,
	Sep		=> 9,
	Oct		=> 10,
	Nov		=> 11,
	Dec		=> 12,
	Q1		=> 3,
	Q2		=> 6,
	Q3		=> 9,
	Q4		=> 12,
);

sub gro {
	my ($lds,undef) = @_;
	$lds =~ /GRO (\d+\w? \d+)/;
	my $gro = $1;
	return ($gro);
}

sub no_conflict {
	my ($a,$b) = @_;
	my $conflict = (
		($mo_fname{$a} && $mo_fname{$b} && ($mo_fname{$a} ne $mo_fname{$b}))
     || ($mo_sname{$a} && $mo_sname{$b} && ($mo_sname{$a} ne $mo_sname{$b}))
	 || ($fa_fname{$a} && $fa_fname{$b} && ($fa_fname{$a} ne $fa_fname{$b}))
     || ($fa_sname{$a} && $fa_sname{$b} && ($fa_sname{$a} ne $fa_sname{$b}))
     || (($child{$a} eq 'm.') && ($child{$b} eq 'm.') && &gro($lds{$a})
     	 && (&gro($lds{$a}) ne &gro($lds{$b})))
	);
	return(!$conflict);
}

###############  SORT ORDERINGS  ################################

sub by_county {
	$cindex{$chapman{$a}} <=> $cindex{$chapman{$b}}; 
}

sub by_area {$area{$a} <=> $area{$b};}
	
sub by_parish {
	std_parish($parish{$a}) cmp std_parish($parish{$b});
}

sub by_decade {int($date{$a}/10.0) <=> int($date{$b}/10.0); }

sub by_year {
	my ($yra,$yrb);
	($yra = $year{$a}) =~ s/-\d*//;
	($yrb = $year{$b}) =~ s/-\d*//;
	$yra <=> $yrb; 
}

sub by_month {$mindex{$mon{$a}} <=> $mindex{$mon{$b}}; }

sub by_day {$day{$a} <=> $day{$b}; }

sub by_date {$date{$a} <=> $date{$b}}

sub by_batch {$batch{$b} cmp $batch{$a};}

sub by_lds {($lds{$a} cmp $lds{$b});}

sub initial {
	my ($word,undef) = @_;
	return uc(substr($word,0,1));
}

sub by_fa_sname {$fa_sname{$a} cmp $fa_sname{$b}}
sub by_fa_s1name {initial($fa_sname{$a}) cmp initial($fa_sname{$b})}
sub by_fa_fname {$fa_fname{$a} cmp $fa_fname{$b}}
sub by_fa_f1name {initial($fa_fname{$a}) cmp initial($fa_fname{$b})}
sub by_mo_sname {$mo_sname{$a} cmp $mo_sname{$b}}
sub by_mo_s1name {initial($mo_sname{$a}) cmp initial($mo_sname{$b})}
sub by_mo_fname {$mo_fname{$a} cmp $mo_fname{$b}}
sub by_mo_f1name {initial($mo_fname{$a}) cmp initial($mo_fname{$b})}
sub by_ch_sname {$ch_sname{$a} cmp $ch_sname{$b}}
sub by_ch_s1name {initial($ch_sname{$a}) cmp initial($ch_sname{$b})}
sub by_ch_fname {$ch_fname{$a} cmp $ch_fname{$b}}
sub by_ch_f1name {initial($ch_fname{$a}) cmp initial($ch_fname{$b})}

sub by_mother {$mother{$a} cmp $mother{$b}}

sub unblank {
	($name,undef) = @_;
	for ($name) {
		s/^\s*//;
		s/\s*$//;
		$_ ||= 'zzzzz';
	}
	return $name;
}

sub by_fa_sname2 {unblank($fa_sname{$a}) cmp unblank($fa_sname{$b})}
sub by_fa_s1name2 {unblank(initial($fa_sname{$a})) cmp unblank(initial($fa_sname{$b}))}
sub by_fa_fname2 {unblank($fa_fname{$a}) cmp unblank($fa_fname{$b})}
sub by_fa_f1name2 {unblank(initial($fa_fname{$a})) cmp unblank(initial($fa_fname{$b}))}
sub by_mo_sname2 {unblank($mo_sname{$a}) cmp unblank($mo_sname{$b})}
sub by_mo_s1name2 {unblank(initial($mo_sname{$a})) cmp unblank(initial($mo_sname{$b}))}
sub by_mo_fname2 {unblank($mo_fname{$a}) cmp unblank($mo_fname{$b})}
sub by_mo_f1name2 {unblank(initial($mo_fname{$a})) cmp unblank(initial($mo_fname{$b}))}
sub by_ch_sname2 {unblank($ch_sname{$a}) cmp unblank($ch_sname{$b})}
sub by_ch_s1name2 {unblank(initial($ch_sname{$a})) cmp unblank(initial($ch_sname{$b}))}
sub by_ch_fname2 {unblank($ch_fname{$a}) cmp unblank($ch_fname{$b})}
sub by_ch_f1name2 {unblank(initial($ch_fname{$a})) cmp unblank(initial($ch_fname{$b}))}

sub by_input_order {$a <=> $b}

1;

